@@ -1,5 +1,10 @@
+import importlib.util
+import io
+import sys
+from contextlib import redirect_stdout
 from functools import lru_cache
-from typing import Dict, List, Iterable, Any, Union
+from pathlib import Path
+from typing import Dict, List, Iterable, Any, Union, Optional
 
 import numpy as np
 import numpy.typing as npt
@@ -12,23 +17,55 @@ from code_loader.contract.enums import DataStateEnum, TestingSectionEnum, DataSt
 from code_loader.contract.exceptions import DatasetScriptException
 from code_loader.contract.responsedataclasses import DatasetIntegParseResult, DatasetTestResultPayload, \
     DatasetPreprocess, DatasetSetup, DatasetInputInstance, DatasetOutputInstance, DatasetMetadataInstance, \
-    VisualizerInstance, PredictionTypeInstance, ModelSetup, CustomLayerInstance, MetricInstance
+    VisualizerInstance, PredictionTypeInstance, ModelSetup, CustomLayerInstance, MetricInstance, CustomLossInstance
 from code_loader.leap_binder import global_leap_binder
 from code_loader.utils import get_root_exception_line_number, get_shape
 
 
 class LeapLoader:
-    def __init__(self, dataset_script: str):
+    def __init__(self, dataset_script: str, code_path: str = '', code_entry_name: str = ''):
         self.dataset_script: str = dataset_script
+        self.code_entry_name = code_entry_name
+        self.code_path = code_path
 
     @lru_cache()
     def exec_script(self) -> None:
         try:
-            global_variables: Dict[Any, Any] = {}
-            exec(self.dataset_script, global_variables)
+            # TODO: make these fields mandatory and delete load_script method
+            if self.code_path and self.code_entry_name:
+                self.evaluate_module()
+            else:
+                self.load_script()
+
         except Exception as e:
             raise DatasetScriptException(getattr(e, 'message', repr(e))) from e
 
+    def load_script(self) -> None:
+        global_variables: Dict[Any, Any] = {}
+        exec(self.dataset_script, global_variables)
+
+    def evaluate_module(self) -> None:
+        def append_path_recursively(full_path: str) -> None:
+            if '/' not in full_path or full_path == '/':
+                return
+
+            parent_path = str(Path(full_path).parent)
+            append_path_recursively(parent_path)
+            sys.path.append(parent_path)
+
+        append_path_recursively(self.code_path)
+
+        file_path = Path(self.code_path, self.code_entry_name)
+        spec = importlib.util.spec_from_file_location(self.code_path, file_path)
+        if spec is None or spec.loader is None:
+            raise DatasetScriptException(f'Something is went wrong with spec file from: {file_path}')
+
+        file = importlib.util.module_from_spec(spec)
+        if file is None:
+            raise DatasetScriptException(f'Something is went wrong with import module from: {file_path}')
+
+        spec.loader.exec_module(file)
+
     @lru_cache()
     def metric_by_name(self) -> Dict[str, MetricHandler]:
         self.exec_script()
@@ -83,22 +120,33 @@ class LeapLoader:
         test_payloads: List[DatasetTestResultPayload] = []
         setup_response = None
         general_error = None
-        try:
-            self.exec_script()
-            preprocess_test_payload = self._check_preprocess()
-            test_payloads.append(preprocess_test_payload)
-            handlers_test_payloads = self._check_handlers()
-            test_payloads.extend(handlers_test_payloads)
-            is_valid = all([payload.is_passed for payload in test_payloads])
-            setup_response = self.get_dataset_setup_response()
-        except DatasetScriptException as e:
-            line_number = get_root_exception_line_number()
-            general_error = f"Something went wrong, {repr(e.__cause__)} line number: {line_number}"
-            is_valid = False
-        except Exception as e:
-            line_number = get_root_exception_line_number()
-            general_error = f"Something went wrong, {repr(e)} line number: {line_number}"
-            is_valid = False
+        stdout_steam = io.StringIO()
+        with redirect_stdout(stdout_steam):
+            try:
+                self.exec_script()
+                preprocess_test_payload = self._check_preprocess()
+                test_payloads.append(preprocess_test_payload)
+                handlers_test_payloads = self._check_handlers()
+                test_payloads.extend(handlers_test_payloads)
+                is_valid = all([payload.is_passed for payload in test_payloads])
+                setup_response = self.get_dataset_setup_response()
+            except DatasetScriptException as e:
+                line_number = get_root_exception_line_number()
+                general_error = f"Something went wrong, {repr(e.__cause__)} line number: {line_number}"
+                is_valid = False
+            except Exception as e:
+                line_number = get_root_exception_line_number()
+                general_error = f"Something went wrong, {repr(e)} line number: {line_number}"
+                is_valid = False
+
+        print_log = stdout_steam.getvalue()
+        if print_log is not None and print_log != "":
+            if general_error is None:
+                general_error = ""
+            else:
+                general_error += "\n\n"
+            general_error += "Print Log:\n"
+            general_error += print_log
 
         is_valid_for_model = bool(global_leap_binder.setup_container.custom_layers)
         model_setup = self.get_model_setup_response()
@@ -215,15 +263,12 @@ class LeapLoader:
             VisualizerInstance(visualizer_handler.name, visualizer_handler.type, visualizer_handler.arg_names)
             for visualizer_handler in setup.visualizers]
 
-        custom_loss_names = [custom_loss.name for custom_loss in setup.custom_loss_handlers]
+        custom_losses = [CustomLossInstance(custom_loss.name, custom_loss.arg_names)
+                         for custom_loss in setup.custom_loss_handlers]
 
         prediction_types = []
         for prediction_type in setup.prediction_types:
-            custom_metrics_names = None
-            if prediction_type.custom_metrics:
-                custom_metrics_names = [custom_metric.__name__ for custom_metric in prediction_type.custom_metrics]
-            pred_type_inst = PredictionTypeInstance(prediction_type.name, prediction_type.labels,
-                                                    prediction_type.metrics, custom_metrics_names)
+            pred_type_inst = PredictionTypeInstance(prediction_type.name, prediction_type.labels)
             prediction_types.append(pred_type_inst)
 
         metrics = []
@@ -233,7 +278,7 @@ class LeapLoader:
 
         return DatasetSetup(preprocess=dataset_preprocess, inputs=inputs, outputs=ground_truths, metadata=metadata,
                             visualizers=visualizers, prediction_types=prediction_types,
-                            custom_loss_names=custom_loss_names, metrics=metrics)
+                            custom_losses=custom_losses, metrics=metrics)
 
     @staticmethod
     def get_model_setup_response() -> ModelSetup:
